# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to update connection profiles for datastream."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.datastream import connection_profiles
from googlecloudsdk.api_lib.datastream import util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.datastream import flags
from googlecloudsdk.command_lib.datastream import resource_args
from googlecloudsdk.command_lib.datastream.connection_profiles import flags as cp_flags
from googlecloudsdk.core.console import console_io

DESCRIPTION = ('Updates a Datastream connection profile')
EXAMPLES = """\
    To update a connection profile for Oracle:

        $ {command} CONNECTION_PROFILE --location=us-central1 --type=oracle --oracle-password=fakepassword --oracle-username=fakeuser --display-name=my-profile --oracle-hostname=35.188.150.50 --oracle-port=1521 --database-service=ORCL --static-ip-connectivity

    To update a connection profile for MySQL:

        $ {command} CONNECTION_PROFILE --location=us-central1 --type=mysql --mysql-password=fakepassword --mysql-username=fakeuser --display-name=my-profile --mysql-hostname=35.188.150.50 --mysql-port=3306 --static-ip-connectivity

    To update a connection profile for PostgreSQL:

        $ {command} CONNECTION_PROFILE --location=us-central1 --type=postgresql --postgresql-password=fakepassword --postgresql-username=fakeuser --display-name=my-profile --postgresql-hostname=35.188.150.50 --postgresql-port=5432 --postgresql-database=db --static-ip-connectivity

    To update a connection profile for Google Cloud Storage:

        $ {command} CONNECTION_PROFILE --location=us-central1 --type=google-cloud-storage --bucket=fake-bucket --root-path=/root/path --display-name=my-profile

    To update a connection profile for Google Cloud Storage:

        $ {command} CONNECTION_PROFILE --location=us-central1 --type=bigquery --display-name=my-profile
   """

EXAMPLES_BETA = """\
    To update a connection profile for Oracle:

        $ {command} CONNECTION_PROFILE --location=us-central1 --type=oracle --oracle-password=fakepassword --oracle-username=fakeuser --display-name=my-profile --oracle-hostname=35.188.150.50 --oracle-port=1521 --database-service=ORCL --static-ip-connectivity

    To update a connection profile for MySQL:

        $ {command} CONNECTION_PROFILE --location=us-central1 --type=mysql --mysql-password=fakepassword --mysql-username=fakeuser --display-name=my-profile --mysql-hostname=35.188.150.50 --mysql-port=3306 --static-ip-connectivity

    To update a connection profile for Google Cloud Storage:

        $ {command} CONNECTION_PROFILE --location=us-central1 --type=google-cloud-storage --bucket-name=fake-bucket --root-path=/root/path --display-name=my-profile --no-connectivity
   """


@base.ReleaseTracks(base.ReleaseTrack.GA)
class Update(base.Command):
  """Update a Datastream connection profile."""
  detailed_help = {'DESCRIPTION': DESCRIPTION, 'EXAMPLES': EXAMPLES}

  @staticmethod
  def CommonArgs(parser, release_track):
    """Common arguments for all release tracks.

    Args:
      parser: An argparse parser that you can use to add arguments that go on
        the command line after this command. Positional arguments are allowed.
      release_track: Some arguments are added based on the command release
        track.
    """
    resource_args.AddConnectionProfileResourceArg(
        parser, 'to update', release_track, required=False)

    cp_flags.AddTypeFlag(parser)
    cp_flags.AddDisplayNameFlag(parser, required=False)
    if release_track == base.ReleaseTrack.GA:
      cp_flags.AddValidationGroup(parser, 'Update')

    profile_flags = parser.add_group(mutex=True)
    cp_flags.AddMysqlProfileGroup(profile_flags, required=False)
    cp_flags.AddOracleProfileGroup(profile_flags, required=False)
    cp_flags.AddPostgresqlProfileGroup(profile_flags, required=False)
    cp_flags.AddGcsProfileGroup(profile_flags, release_track, required=False)
    flags.AddLabelsUpdateFlags(parser)

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command."""
    Update.CommonArgs(parser, base.ReleaseTrack.GA)

  def Run(self, args):
    """Update a Datastream connection profile.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
        with.

    Returns:
      A dict object representing the operations resource describing the update
      operation if the update was successful.
    """
    connection_profile_ref = args.CONCEPTS.connection_profile.Parse()

    if args.oracle_prompt_for_password:
      args.oracle_password = console_io.PromptPassword(
          'Please Enter Password: ')

    if args.mysql_prompt_for_password:
      args.mysql_password = console_io.PromptPassword('Please Enter Password: ')

    if args.postgresql_prompt_for_password:
      args.postgresql_password = console_io.PromptPassword(
          'Please Enter Password: ')

    cp_type = (args.type).upper()
    cp_client = connection_profiles.ConnectionProfilesClient()
    result_operation = cp_client.Update(connection_profile_ref.RelativeName(),
                                        cp_type, self.ReleaseTrack(), args)

    client = util.GetClientInstance()
    messages = util.GetMessagesModule()
    resource_parser = util.GetResourceParser()

    operation_ref = resource_parser.Create(
        'datastream.projects.locations.operations',
        operationsId=result_operation.name,
        projectsId=connection_profile_ref.projectsId,
        locationsId=connection_profile_ref.locationsId)

    return client.projects_locations_operations.Get(
        messages.DatastreamProjectsLocationsOperationsGetRequest(
            name=operation_ref.operationsId))


@base.Deprecate(
    is_removed=False,
    warning=('Datastream beta version is deprecated. Please use`gcloud '
             'datastream connection-profiles update` command instead.')
)
@base.ReleaseTracks(base.ReleaseTrack.BETA)
class UpdateBeta(Update):
  """Update a Datastream connection profile."""
  detailed_help = {'DESCRIPTION': DESCRIPTION, 'EXAMPLES': EXAMPLES_BETA}

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command."""
    Update.CommonArgs(parser, base.ReleaseTrack.BETA)



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to tail logs for a job."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.logs import read as read_logs_lib
from googlecloudsdk.command_lib.run import flags
from googlecloudsdk.command_lib.run import streaming
from googlecloudsdk.core import properties


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
class Tail(base.BinaryBackedCommand):
  """Tail logs for Cloud Run jobs."""

  detailed_help = {
      'DESCRIPTION':
          """\
          {command} tails log-entries for a particular
          Cloud Run job in real time.  The log entries are formatted for
          consumption in a terminal.
          """,
      'EXAMPLES':
          """\
          To tail log entries for a Cloud Run job, run:

            $ {command} my-job

          To tail log entries with severity ERROR or higher, run:

            $ {command} my-job --log-filter="severity>=ERROR"

          Detailed information about filters can be found at:
          [](https://cloud.google.com/logging/docs/view/advanced_filters)
          """,
  }

  @staticmethod
  def Args(parser):
    parser.add_argument('job', help='Name for a Cloud Run job.')
    read_logs_lib.LogFilterArgs(parser)

  def Run(self, args):
    filters = []
    if args.IsSpecified('log_filter'):
      filters.append(args.log_filter)
    filters.append('resource.type=%s' % 'cloud_run_job')
    filters.append('resource.labels.job_name=%s' % args.job)
    filters.append('resource.labels.location=%s' %
                   flags.GetRegion(args, prompt=True))
    filters.append('severity>=DEFAULT')
    project_id = properties.VALUES.core.project.Get(required=True)
    filter_str = ' '.join(filters)
    command_executor = streaming.LogStreamingWrapper()
    response = command_executor(
        project_id=project_id, log_format='run', log_filter=filter_str)
    return self._DefaultOperationResponseHandler(response)



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to show fleet information."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.container.fleet import client
from googlecloudsdk.api_lib.container.fleet import util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.container.fleet import resources


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
class Describe(base.DescribeCommand):
  """Show fleet namespace RBAC RoleBinding information.

  This command can fail for the following reasons:
  * The RoleBinding specified does not exist in the project.
  * The caller does not have permission to access the RoleBinding.

  ## EXAMPLES

  To print metadata for RBAC RoleBinding `RBRB` in the namespace `NAMESPACE`,
  run:

    $ {command} RBRB --namespace=NAMESPACE
  """

  @classmethod
  def Args(cls, parser):
    resources.AddRBACResourceArg(
        parser,
        api_version=util.VERSION_MAP[cls.ReleaseTrack()],
        rbacrb_help=(
            'Name of the RBAC RoleBinding to be created. '
            'Must comply with RFC 1123 (up to 63 characters, '
            "alphanumeric and '-')"
        ),
    )

  def Run(self, args):
    fleetclient = client.FleetClient(release_track=self.ReleaseTrack())
    return fleetclient.GetRBACRoleBinding(resources.RBACResourceName(args))



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""List the keys within a keyring."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.kmsinventory import inventory
from googlecloudsdk.calliope import base

from googlecloudsdk.core import properties

DETAILED_HELP = {
    'DESCRIPTION': """
         *{command}* lists the keys in the specified project.
       """,
    'EXAMPLES': """
        To view the keys in the default project, run:

           $ {command}

        To view the keys in project `jellyfish`, run:

           $ {command} --project=jellyfish
       """,
}


class ListKeys(base.ListCommand):
  """Lists the keys in a project."""
  detailed_help = DETAILED_HELP

  @staticmethod
  def Args(parser):
    pass

  def Run(self, args):
    # NOMUTANTS--no good way to test
    project = properties.VALUES.core.project.Get(required=True)
    return inventory.ListKeys(project, args)



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""'Bare Metal Solution boot volumes "restore" command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.bms.bms_client import BmsClient
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.bms import flags
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources


DETAILED_HELP = {
    'DESCRIPTION':
        """
          Restore a Bare Metal Solution boot volume from an existing snapshot.
        """,
    'EXAMPLES':
        """
          To restore a boot volume named ``my-boot-volume'' in region
          ``us-central1'' from snapshot named ``my-snapshot'', run:

          $ {command} my-boot-volume --region=us-central1 --snapshot=my-snapshot
    """,
}


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.GA)
class Restore(base.UpdateCommand):
  """Restore a Bare Metal Solution boot volume from an existing snapshot."""

  @staticmethod
  def Args(parser):
    """Register flags for this command."""
    flags.AddVolumeArgToParser(parser, positional=True)
    parser.add_argument('--snapshot',
                        required=True,
                        help='Name of the snapshot to restore.')
    base.ASYNC_FLAG.AddToParser(parser)

  def Run(self, args):
    volume = args.CONCEPTS.volume.Parse()
    client = BmsClient()
    snapshot_ref = resources.REGISTRY.Parse(
        args.snapshot,
        params={
            'projectsId': properties.VALUES.core.project.GetOrFail,
            'locationsId': args.region,
            'volumesId': args.volume,
        },
        collection='baremetalsolution.projects.locations.volumes.snapshots',
        api_version='v2')
    op_ref = client.RestoreVolumeSnapshot(
        snapshot_name=snapshot_ref.RelativeName())

    if op_ref.done:
      log.RestoredResource(volume.Name(), kind='volume')
      return op_ref

    if args.async_:
      log.status.Print('Restore request issued for [{}]\nCheck operation '
                       '[{}] for status.'.format(volume.Name(), op_ref.name))
      return op_ref

    op_resource = resources.REGISTRY.ParseRelativeName(
        op_ref.name,
        collection='baremetalsolution.projects.locations.operations',
        api_version='v2')
    poller = waiter.CloudOperationPollerNoResources(
        client.operation_service)
    res = waiter.WaitFor(poller, op_resource,
                         'Waiting for operation [{}] to complete'.format(
                             op_ref.name))
    log.RestoredResource(volume.Name(), kind='volume')
    return res


Restore.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Utilities for dealing with region resources."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals


class Region(object):
  """Value class representing a region resource."""

  def __init__(self, region, standard, flexible, search_api):
    self.region = region
    self.standard = standard
    self.flexible = flexible
    self.search_api = search_api

  @classmethod
  def FromRegionResource(cls, region):
    """Create region from a google.cloud.location.Location message."""
    flex = False
    standard = False
    search_api = False
    region_id = region.labels.additionalProperties[0].value
    for p in region.metadata.additionalProperties:
      if p.key == 'flexibleEnvironmentAvailable' and p.value.boolean_value:
        flex = True
      elif p.key == 'standardEnvironmentAvailable' and p.value.boolean_value:
        standard = True
      elif p.key == 'searchApiAvailable' and p.value.boolean_value:
        search_api = True

    return cls(region_id, standard, flex, search_api)

  def __str__(self):
    envs = (
        x[1] for x in [(self.standard, 'standard'),
                       (self.flexible, 'flexible'),
                       (self.search_api, 'search_api')] if x[0])
    out = '{region: <13}'.format(region=self.region)
    return out + ' (supports {envs})'.format(envs=' and '.join(envs))



"""Generated message classes for anthospolicycontrollerstatus_pa version v1alpha.

anthospolicycontrollerstatus-pa.googleapis.com API.
"""
# NOTE: This file is autogenerated and should not be edited by hand.

from __future__ import absolute_import

from apitools.base.protorpclite import messages as _messages
from apitools.base.py import encoding
from apitools.base.py import extra_types


package = 'anthospolicycontrollerstatus_pa'


class AnthospolicycontrollerstatusPaProjectsFleetConstraintTemplatesGetRequest(_messages.Message):
  r"""A
  AnthospolicycontrollerstatusPaProjectsFleetConstraintTemplatesGetRequest
  object.

  Fields:
    name: Required. The name of the fleet constraint template to retrieve.
      Format: projects/{project_id}/fleetConstraintTemplates/{constraint_templ
      ate_name}.
  """

  name = _messages.StringField(1, required=True)


class AnthospolicycontrollerstatusPaProjectsFleetConstraintTemplatesListRequest(_messages.Message):
  r"""A
  AnthospolicycontrollerstatusPaProjectsFleetConstraintTemplatesListRequest
  object.

  Fields:
    pageSize: The maximum number of fleet constraint templates to return. The
      service may return fewer than this value. If unspecified or 0, defaults
      to 500 results. The maximum value is 2000; values above 2000 will be
      coerced to 2000.
    pageToken: A page token, received from a previous
      ListFleetConstraintTemplates call. Provide this to retrieve the
      subsequent page. When paginating, all other parameters provided to
      ListFleetConstraintTemplates must match the call that provided the page
      token.
    parent: Required. The project id for which to fetch fleet constraint
      templates. Format: projects/{project_id}.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class AnthospolicycontrollerstatusPaProjectsFleetConstraintsGetRequest(_messages.Message):
  r"""A AnthospolicycontrollerstatusPaProjectsFleetConstraintsGetRequest
  object.

  Fields:
    name: Required. The name of the fleet constraint to retrieve. Format: proj
      ects/{project_id}/fleetConstraints/{constraint_template_name}/{constrain
      t_name}.
  """

  name = _messages.StringField(1, required=True)


class AnthospolicycontrollerstatusPaProjectsFleetConstraintsListRequest(_messages.Message):
  r"""A AnthospolicycontrollerstatusPaProjectsFleetConstraintsListRequest
  object.

  Fields:
    pageSize: The maximum number of fleet constraints to return. The service
      may return fewer than this value. If unspecified or 0, defaults to 500
      results. The maximum value is 2000; values above 2000 will be coerced to
      2000.
    pageToken: A page token, received from a previous ListFleetConstraints
      call. Provide this to retrieve the subsequent page. When paginating, all
      other parameters provided to ListFleetConstraints must match the call
      that provided the page token.
    parent: Required. The project id for which to fetch fleet constraints.
      Format: projects/{project_id}.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class AnthospolicycontrollerstatusPaProjectsMembershipConstraintAuditViolationsListRequest(_messages.Message):
  r"""A AnthospolicycontrollerstatusPaProjectsMembershipConstraintAuditViolati
  onsListRequest object.

  Fields:
    pageSize: The maximum number of membership constraint audit violations to
      return. The service may return fewer than this value. If unspecified or
      0, defaults to 500 results. The maximum value is 2000; values above 2000
      will be coerced to 2000.
    pageToken: A page token, received from a previous
      ListMembershipConstraintAuditViolations call. Provide this to retrieve
      the subsequent page. When paginating, all other parameters provided to
      ListMembershipConstraintAuditViolations must match the call that
      provided the page token.
    parent: Required. The project id for which to fetch membership constraint
      audit violations. Format: projects/{project_id}.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class AnthospolicycontrollerstatusPaProjectsMembershipConstraintTemplatesGetRequest(_messages.Message):
  r"""A AnthospolicycontrollerstatusPaProjectsMembershipConstraintTemplatesGet
  Request object.

  Fields:
    name: Required. The name of the membership constraint template to
      retrieve. Format: projects/{project_id}/membershipConstraintTemplates/{c
      onstraint_template_name}/{membership_id}.
  """

  name = _messages.StringField(1, required=True)


class AnthospolicycontrollerstatusPaProjectsMembershipConstraintTemplatesListRequest(_messages.Message):
  r"""A AnthospolicycontrollerstatusPaProjectsMembershipConstraintTemplatesLis
  tRequest object.

  Fields:
    pageSize: The maximum number of membership constraint templates to return.
      The service may return fewer than this value. If unspecified or 0,
      defaults to 500 results. The maximum value is 2000; values above 2000
      will be coerced to 2000.
    pageToken: A page token, received from a previous
      ListMembershipConstraintTemplates call. Provide this to retrieve the
      subsequent page. When paginating, all other parameters provided to
      ListMembershipConstraintTemplates must match the call that provided the
      page token.
    parent: Required. The project id for which to fetch membership constraint
      templates. Format: projects/{project_id}.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class AnthospolicycontrollerstatusPaProjectsMembershipConstraintsGetRequest(_messages.Message):
  r"""A AnthospolicycontrollerstatusPaProjectsMembershipConstraintsGetRequest
  object.

  Fields:
    name: Required. The name of the membership constraint to retrieve. Format:
      projects/{project_id}/membershipConstraints/{constraint_template_name}/{
      constraint_name}/{membership_id}.
  """

  name = _messages.StringField(1, required=True)


class AnthospolicycontrollerstatusPaProjectsMembershipConstraintsListRequest(_messages.Message):
  r"""A AnthospolicycontrollerstatusPaProjectsMembershipConstraintsListRequest
  object.

  Fields:
    pageSize: The maximum number of membership constraints to return. The
      service may return fewer than this value. If unspecified or 0, defaults
      to 500 results. The maximum value is 2000; values above 2000 will be
      coerced to 2000.
    pageToken: A page token, received from a previous
      ListMembershipConstraints call. Provide this to retrieve the subsequent
      page. When paginating, all other parameters provided to
      ListMembershipConstraints must match the call that provided the page
      token.
    parent: Required. The project id for which to fetch membership
      constraints. Format: projects/{project_id}.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class AnthospolicycontrollerstatusPaProjectsMembershipsListRequest(_messages.Message):
  r"""A AnthospolicycontrollerstatusPaProjectsMembershipsListRequest object.

  Fields:
    pageSize: The maximum number of memberships to return. The service may
      return fewer than this value. If unspecified or 0, defaults to 500
      results. The maximum value is 2000; values above 2000 will be coerced to
      2000.
    pageToken: A page token, received from a previous ListMemberships call.
      Provide this to retrieve the subsequent page. When paginating, all other
      parameters provided to ListMemberships must match the call that provided
      the page token.
    parent: Required. The project id for which to fetch memberships' policy
      controller status. Format: projects/{project_id}.
  """

  pageSize = _messages.IntegerField(1, variant=_messages.Variant.INT32)
  pageToken = _messages.StringField(2)
  parent = _messages.StringField(3, required=True)


class ConstraintInfo(_messages.Message):
  r"""A ConstraintInfo object.

  Messages:
    InfoValue: A InfoValue object.

  Fields:
    info: A InfoValue attribute.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class InfoValue(_messages.Message):
    r"""A InfoValue object.

    Messages:
      AdditionalProperty: An additional property for a InfoValue object.

    Fields:
      additionalProperties: Additional properties of type InfoValue
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a InfoValue object.

      Fields:
        key: Name of the additional property.
        value: A string attribute.
      """

      key = _messages.StringField(1)
      value = _messages.StringField(2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  info = _messages.MessageField('InfoValue', 1)


class ConstraintRef(_messages.Message):
  r"""Constraint represents a single constraint. Base identifying resource.

  Fields:
    constraintTemplateName: The constraint template name, lowercase of the
      constraint kind. Used for identification, not for UI display.
    name: The constraint name.
  """

  constraintTemplateName = _messages.StringField(1)
  name = _messages.StringField(2)


class ConstraintTemplateRef(_messages.Message):
  r"""ConstraintTemplateRef identifies a constraint template.

  Fields:
    name: The constraint template name.
  """

  name = _messages.StringField(1)


class FleetConstraint(_messages.Message):
  r"""The fleet-wide info for a constraint. Includes the number of constraints
  to avoid clients making serial round trips.

  Fields:
    numMemberships: The number of memberships where this constraint was found.
    numViolations: The number of violations of this constraint.
    ref: The constraint this data refers to.
  """

  numMemberships = _messages.IntegerField(1)
  numViolations = _messages.IntegerField(2)
  ref = _messages.MessageField('ConstraintRef', 3)


class FleetConstraintTemplate(_messages.Message):
  r"""FleetConstraintTemplate contains aggregate status for a single
  constraint template. The aggregation is across all member clusters in a
  fleet.

  Fields:
    numConstraints: The number of unique constraints using this constraint
      template. This is included so avoid clients don't have to make serial
      round trips.
    numMemberships: The number of member clusters on which this constraint
      template was found.
    ref: The constraint template this data refers to.
  """

  numConstraints = _messages.IntegerField(1)
  numMemberships = _messages.IntegerField(2)
  ref = _messages.MessageField('ConstraintTemplateRef', 3)


class GroupKind(_messages.Message):
  r"""GroupKind includes the group, kind of the K8s resource.

  Fields:
    apiGroup: The api group of the resource.
    kind: The api kind of the resource.
  """

  apiGroup = _messages.StringField(1)
  kind = _messages.StringField(2)


class KubernetesMatch(_messages.Message):
  r"""The scope of objects to which a given constraint will be applied

  Enums:
    ScopeValueValuesEnum: Matcher to match on scope of objects.

  Fields:
    excludedNamespaces: Matcher to match on objects not in excluded
      namespaces. Supports a prefix-based glob.
    groupKinds: Matcher to match on objects based on api group or kind.
    labelSelector: Matcher to match objects based on label keys or values.
    name: Matcher to match on an object's name. Supports a prefix-based glob.
    namespaceSelector: Matcher to match on namespace.
    namespaces: Matcher to match on objects in given namespaces. Supports a
      prefix-based glob.
    scope: Matcher to match on scope of objects.
  """

  class ScopeValueValuesEnum(_messages.Enum):
    r"""Matcher to match on scope of objects.

    Values:
      SCOPE_UNSPECIFIED: Unspecified scope.
      SCOPE_ALL: Scope `*`, match all resources.
      SCOPE_CLUSTER: Scope `Cluster`, match cluster-scoped resources.
      SCOPE_NAMESPACED: Scope `Namespaced`, match namespace-scoped resources.
    """
    SCOPE_UNSPECIFIED = 0
    SCOPE_ALL = 1
    SCOPE_CLUSTER = 2
    SCOPE_NAMESPACED = 3

  excludedNamespaces = _messages.StringField(1, repeated=True)
  groupKinds = _messages.MessageField('GroupKind', 2, repeated=True)
  labelSelector = _messages.StringField(3)
  name = _messages.StringField(4)
  namespaceSelector = _messages.StringField(5)
  namespaces = _messages.StringField(6, repeated=True)
  scope = _messages.EnumField('ScopeValueValuesEnum', 7)


class ListFleetConstraintTemplatesResponse(_messages.Message):
  r"""Response schema for ListFleetConstraintTemplates.

  Fields:
    fleetConstraintTemplates: List of fleet-wide constraint template info.
    nextPageToken: A token, which can be sent as page_token to retrieve the
      next page. If this field is omitted, there are no subsequent pages.
    totalSize: The number of fleet constraint templates in the response.
  """

  fleetConstraintTemplates = _messages.MessageField('FleetConstraintTemplate', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  totalSize = _messages.IntegerField(3)


class ListFleetConstraintsResponse(_messages.Message):
  r"""Response schema for ListFleetConstraints.

  Fields:
    fleetConstraints: List of fleet-wide constraint info.
    nextPageToken: A token, which can be sent as page_token to retrieve the
      next page. If this field is omitted, there are no subsequent pages.
    totalSize: The number of fleet constraints in the response.
  """

  fleetConstraints = _messages.MessageField('FleetConstraint', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  totalSize = _messages.IntegerField(3)


class ListMembershipConstraintAuditViolationsResponse(_messages.Message):
  r"""Response schema for ListMembershipConstraintAuditViolations.

  Fields:
    membershipConstraintAuditViolations: List of the membership-level
      constraint audit violation info.
    nextPageToken: A token, which can be sent as page_token to retrieve the
      next page. If this field is omitted, there are no subsequent pages.
    totalSize: The number of membership constraint audit violations in the
      response.
  """

  membershipConstraintAuditViolations = _messages.MessageField('MembershipConstraintAuditViolation', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  totalSize = _messages.IntegerField(3)


class ListMembershipConstraintTemplatesResponse(_messages.Message):
  r"""Response schema for ListMembershipConstraintTemplates.

  Fields:
    membershipConstraintTemplates: List of membership-level constraint
      template info.
    nextPageToken: A token, which can be sent as page_token to retrieve the
      next page. If this field is omitted, there are no subsequent pages.
    totalSize: The number of membership constraint templates in the response.
  """

  membershipConstraintTemplates = _messages.MessageField('MembershipConstraintTemplate', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  totalSize = _messages.IntegerField(3)


class ListMembershipConstraintsResponse(_messages.Message):
  r"""Response schema for ListMembershipConstraints.

  Fields:
    membershipConstraints: List of membership-level constraint info.
    nextPageToken: A token, which can be sent as page_token to retrieve the
      next page. If this field is omitted, there are no subsequent pages.
    totalSize: The number of membership constraints in the response.
  """

  membershipConstraints = _messages.MessageField('MembershipConstraint', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  totalSize = _messages.IntegerField(3)


class ListMembershipsResponse(_messages.Message):
  r"""Response schema for ListMemberships.

  Fields:
    memberships: List of the memberships in a given fleet.
    nextPageToken: A token, which can be sent as page_token to retrieve the
      next page. If this field is omitted, there are no subsequent pages.
    totalSize: The number of memberships in the response.
  """

  memberships = _messages.MessageField('Membership', 1, repeated=True)
  nextPageToken = _messages.StringField(2)
  totalSize = _messages.IntegerField(3)


class Membership(_messages.Message):
  r"""Membership contains aggregate information about policy controller
  resources on a member cluster.

  Fields:
    featureStatus: The status of the policy controller feature.
    ref: The membership this data refers to.
    runtimeStatus: The status of the policy controller runtime configuration.
      If runtime_status is empty, then the server could not find any existing
      constraint or templates to report status.
  """

  featureStatus = _messages.MessageField('MembershipFeatureStatus', 1)
  ref = _messages.MessageField('MembershipRef', 2)
  runtimeStatus = _messages.MessageField('MembershipRuntimeStatus', 3)


class MembershipConstraint(_messages.Message):
  r"""Membership specific constraint data.

  Fields:
    constraintRef: The constraint this data refers to.
    content: The string content for the constraint resource.
    kind: The kind of the constraint on this membership, for display purposes.
    membershipRef: The membership this data refers to.
    metadata: Membership-specific constraint metadata.
    spec: Membership-specific constraint spec.
    status: Membership-specific constraint status.
  """

  constraintRef = _messages.MessageField('ConstraintRef', 1)
  content = _messages.MessageField('StringContent', 2)
  kind = _messages.StringField(3)
  membershipRef = _messages.MessageField('MembershipRef', 4)
  metadata = _messages.MessageField('MembershipConstraintMetadata', 5)
  spec = _messages.MessageField('MembershipConstraintSpec', 6)
  status = _messages.MessageField('MembershipConstraintStatus', 7)


class MembershipConstraintAuditViolation(_messages.Message):
  r"""MembershipConstraintAuditViolation encodes info relevant to a violation
  of a single constraint on a single member cluster.

  Fields:
    auditTimestamp: The audit timestamp when this violation was observed on
      the membership.
    constraintRef: The constraint ref of the violated constraint.
    errorMessage: An error message detailing the violation.
    membershipRef: The membership this violation occurs on.
    resourceRef: The resource ref of the violating K8S object.
  """

  auditTimestamp = _messages.StringField(1)
  constraintRef = _messages.MessageField('ConstraintRef', 2)
  errorMessage = _messages.StringField(3)
  membershipRef = _messages.MessageField('MembershipRef', 4)
  resourceRef = _messages.MessageField('ResourceRef', 5)


class MembershipConstraintMetadata(_messages.Message):
  r"""MembershipConstraintMetadata contains relevant fields from constraint
  metadata.

  Fields:
    constraintInfo: constraint bundle information from the metadata.annotation
      field.
    creation: metadata.creation_timestamp from the constraint.
    generation: metadata.generation from the constraint.
  """

  constraintInfo = _messages.MessageField('ConstraintInfo', 1)
  creation = _messages.StringField(2)
  generation = _messages.IntegerField(3)


class MembershipConstraintSpec(_messages.Message):
  r"""The spec defining this constraint. See https://open-policy-
  agent.github.io/gatekeeper/website/docs/howto#constraints.

  Enums:
    EnforcementActionValueValuesEnum: spec.enforcement_action.

  Fields:
    enforcementAction: spec.enforcement_action.
    kubernetesMatch: Reserved: The match specified against GCP resources.
      GCPMatch gcp_match = 3;
    parameters: The parameters a constraint expects.
  """

  class EnforcementActionValueValuesEnum(_messages.Enum):
    r"""spec.enforcement_action.

    Values:
      ENFORCEMENT_ACTION_UNSPECIFIED: Unspecified state for an enforcement
        action.
      ENFORCEMENT_ACTION_DENY: The resource is denied admission to the
        membership.
      ENFORCEMENT_ACTION_DRYRUN: Allows testing constraints without enforcing
        them.
      ENFORCEMENT_ACTION_WARN: Provides immediate feedback on why a resource
        violates a constraint.
    """
    ENFORCEMENT_ACTION_UNSPECIFIED = 0
    ENFORCEMENT_ACTION_DENY = 1
    ENFORCEMENT_ACTION_DRYRUN = 2
    ENFORCEMENT_ACTION_WARN = 3

  enforcementAction = _messages.EnumField('EnforcementActionValueValuesEnum', 1)
  kubernetesMatch = _messages.MessageField('KubernetesMatch', 2)
  parameters = _messages.MessageField('MembershipConstraintSpecParameters', 3)


class MembershipConstraintSpecParameters(_messages.Message):
  r"""Parameters specified based on the schema defined in the constraint
  template.

  Messages:
    ParametersValue: spec.parameters from the constraint.

  Fields:
    parameters: spec.parameters from the constraint.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class ParametersValue(_messages.Message):
    r"""spec.parameters from the constraint.

    Messages:
      AdditionalProperty: An additional property for a ParametersValue object.

    Fields:
      additionalProperties: Properties of the object.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a ParametersValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  parameters = _messages.MessageField('ParametersValue', 1)


class MembershipConstraintStatus(_messages.Message):
  r"""MembershipConstraintStatus contains high-level information from
  constraint status. Omits violation-level information from constraint status,
  which is in separate violation resources.

  Fields:
    auditTimestamp: status.audit_timestamp from the constraint.
    numViolations: status.total_violations from the constraint.
  """

  auditTimestamp = _messages.StringField(1)
  numViolations = _messages.IntegerField(2)


class MembershipConstraintTemplate(_messages.Message):
  r"""MembershipConstraintTemplate contains runtime status relevant to a
  single constraint template on a single member cluster.

  Fields:
    constraintTemplateRef: The constraint template this data refers to.
    description: annotations.description, may not be populated.
    membershipRef: The membership this data refers to.
    metadata: Membership-specific constraint template metadata.
    spec: Membership-specific constraint template spec.
    status: Membership-specific constraint template status.
  """

  constraintTemplateRef = _messages.MessageField('ConstraintTemplateRef', 1)
  description = _messages.StringField(2)
  membershipRef = _messages.MessageField('MembershipRef', 3)
  metadata = _messages.MessageField('MembershipConstraintTemplateMetadata', 4)
  spec = _messages.MessageField('MembershipConstraintTemplateSpec', 5)
  status = _messages.MessageField('MembershipConstraintTemplateStatus', 6)


class MembershipConstraintTemplateMetadata(_messages.Message):
  r"""MembershipConstraintTemplateMetadata contains relevant fields from
  constraint template metadata.

  Fields:
    creation: metadata.creation_timestamp from the constraint template.
    generation: metadata.generation from the constraint template.
  """

  creation = _messages.StringField(1)
  generation = _messages.IntegerField(2)


class MembershipConstraintTemplateSpec(_messages.Message):
  r"""The spec defining this constraint template. See https://github.com/open-
  policy-agent/frameworks/tree/master/constraint#what-is-a-constraint-
  template.

  Messages:
    PropertiesValue: spec.crd.spec.validation.openAPIV3Schema.

  Fields:
    constraintKind: spec.crd.spec.names.kind.
    properties: spec.crd.spec.validation.openAPIV3Schema.
    targets: spec.targets. Use a list of targets to account for multi-target
      templates.
  """

  @encoding.MapUnrecognizedFields('additionalProperties')
  class PropertiesValue(_messages.Message):
    r"""spec.crd.spec.validation.openAPIV3Schema.

    Messages:
      AdditionalProperty: An additional property for a PropertiesValue object.

    Fields:
      additionalProperties: Properties of the object.
    """

    class AdditionalProperty(_messages.Message):
      r"""An additional property for a PropertiesValue object.

      Fields:
        key: Name of the additional property.
        value: A extra_types.JsonValue attribute.
      """

      key = _messages.StringField(1)
      value = _messages.MessageField('extra_types.JsonValue', 2)

    additionalProperties = _messages.MessageField('AdditionalProperty', 1, repeated=True)

  constraintKind = _messages.StringField(1)
  properties = _messages.MessageField('PropertiesValue', 2)
  targets = _messages.MessageField('Target', 3, repeated=True)


class MembershipConstraintTemplateStatus(_messages.Message):
  r"""MembershipConstratinTemplateStatus contains status information, e.g.
  whether the template has been created on the member cluster.

  Fields:
    created: status.created from the constraint template.
  """

  created = _messages.BooleanField(1)


class MembershipFeatureStatus(_messages.Message):
  r"""MembershipFeatureStatus contains aggregate data about the policy
  controller feature on a cluster that is a member of a fleet.

  Enums:
    LifecycleStatusValueValuesEnum: The lifecycle status of the policy
      controller feature on the membership.

  Fields:
    lifecycleStatus: The lifecycle status of the policy controller feature on
      the membership.
  """

  class LifecycleStatusValueValuesEnum(_messages.Enum):
    r"""The lifecycle status of the policy controller feature on the
    membership.

    Values:
      LIFECYCLE_STATE_UNSPECIFIED: The lifecycle state is unspecified.
      NOT_INSTALLED: Policy Controller (PC) does not exist on the given
        cluster, and no k8s resources of any type that are associated with the
        PC should exist there. The cluster does not possess a membership with
        the Hub Feature controller.
      INSTALLING: The Hub Feature controller possesses a Membership, however
        Policy Controller is not fully installed on the cluster. In this state
        the hub can be expected to be taking actions to install the PC on the
        cluster.
      ACTIVE: Policy Controller (PC) is fully installed on the cluster and in
        an operational mode. In this state the Hub Feature controller will be
        reconciling state with the PC, and the PC will be performing it's
        operational tasks per that software. Entering a READY state requires
        that the hub has confirmed the PC is installed and its pods are
        operational with the version of the PC the Hub Feature controller
        expects.
      UPDATING: Policy Controller (PC) is fully installed, but in the process
        of changing the configuration (including changing the version of PC
        either up and down, or modifying the manifests of PC) of the resources
        running on the cluster. The Hub Feature controller has a Membership,
        is aware of the version the cluster should be running in, but has not
        confirmed for itself that the PC is running with that version.
      DECOMMISSIONING: Policy Controller (PC) may have resources on the
        cluster, but the Hub Feature controller wishes to remove the
        Membership. The Membership still exists.
      CLUSTER_ERROR: Policy Controller (PC) is not operational, and the Hub
        Feature controller is unable to act to make it operational. Entering a
        CLUSTER_ERROR state happens automatically when the PCH determines that
        a PC installed on the cluster is non-operative or that the cluster
        does not meet requirements set for the Hub Feature controller to
        administer the cluster but has nevertheless been given an instruction
        to do so (such as 'install').
      HUB_ERROR: In this state, the PC may still be operational, and only the
        Hub Feature controller is unable to act. The hub should not issue
        instructions to change the PC state, or otherwise interfere with the
        on-cluster resources. Entering a HUB_ERROR state happens automatically
        when the Hub Feature controller determines the hub is in an unhealthy
        state and it wishes to 'take hands off' to avoid corrupting the PC or
        other data.
      SUSPENDED: Policy Controller (PC) is installed but suspended. This means
        that the policies are not enforced, but violations are still recorded
        (through audit).
    """
    LIFECYCLE_STATE_UNSPECIFIED = 0
    NOT_INSTALLED = 1
    INSTALLING = 2
    ACTIVE = 3
    UPDATING = 4
    DECOMMISSIONING = 5
    CLUSTER_ERROR = 6
    HUB_ERROR = 7
    SUSPENDED = 8

  lifecycleStatus = _messages.EnumField('LifecycleStatusValueValuesEnum', 1)


class MembershipRef(_messages.Message):
  r"""Reference to a GKE Hub membership.

  Fields:
    id: The id of the membership, for identity purposes.
    name: The name of the membership, for display purposes.
  """

  id = _messages.StringField(1)
  name = _messages.StringField(2)


class MembershipRuntimeStatus(_messages.Message):
  r"""MembershipRuntimeStatus contains aggregate data about policy controller
  resources on a cluster that is a member of a fleet.

  Fields:
    numConstraintTemplates: The number of constraint templates on the member
      cluster.
    numConstraintViolations: The number of constraint violations on the member
      cluster.
    numConstraints: The number of constraints on the member cluster.
  """

  numConstraintTemplates = _messages.IntegerField(1)
  numConstraintViolations = _messages.IntegerField(2)
  numConstraints = _messages.IntegerField(3)


class RegoPolicy(_messages.Message):
  r"""The rego policy defining this constraint template.

  Fields:
    libs: spec.targets.libs.
    policy: spec.targets.rego.
  """

  libs = _messages.StringField(1, repeated=True)
  policy = _messages.StringField(2)


class ResourceRef(_messages.Message):
  r"""Reference to a K8S resource.

  Fields:
    groupKind: GK is the GroupKind of the K8S resource. This field may be
      empty for errors that are not associated with a specific resource.
    name: The name of the K8S resource.
    resourceNamespace: The namespace of the K8S resource. This field may be
      empty for errors that are associated with a cluster-scoped resource.
      Called resource_namespace because namespace is a C++ keyword.
  """

  groupKind = _messages.MessageField('GroupKind', 1)
  name = _messages.StringField(2)
  resourceNamespace = _messages.StringField(3)


class StandardQueryParameters(_messages.Message):
  r"""Query parameters accepted by all methods.

  Enums:
    FXgafvValueValuesEnum: V1 error format.
    AltValueValuesEnum: Data format for response.

  Fields:
    f__xgafv: V1 error format.
    access_token: OAuth access token.
    alt: Data format for response.
    callback: JSONP
    fields: Selector specifying which fields to include in a partial response.
    key: API key. Your API key identifies your project and provides you with
      API access, quota, and reports. Required unless you provide an OAuth 2.0
      token.
    oauth_token: OAuth 2.0 token for the current user.
    prettyPrint: Returns response with indentations and line breaks.
    quotaUser: Available to use for quota purposes for server-side
      applications. Can be any arbitrary string assigned to a user, but should
      not exceed 40 characters.
    trace: A tracing token of the form "token:<tokenid>" to include in api
      requests.
    uploadType: Legacy upload protocol for media (e.g. "media", "multipart").
    upload_protocol: Upload protocol for media (e.g. "raw", "multipart").
  """

  class AltValueValuesEnum(_messages.Enum):
    r"""Data format for response.

    Values:
      json: Responses with Content-Type of application/json
      media: Media download with context-dependent Content-Type
      proto: Responses with Content-Type of application/x-protobuf
    """
    json = 0
    media = 1
    proto = 2

  class FXgafvValueValuesEnum(_messages.Enum):
    r"""V1 error format.

    Values:
      _1: v1 error format
      _2: v2 error format
    """
    _1 = 0
    _2 = 1

  f__xgafv = _messages.EnumField('FXgafvValueValuesEnum', 1)
  access_token = _messages.StringField(2)
  alt = _messages.EnumField('AltValueValuesEnum', 3, default='json')
  callback = _messages.StringField(4)
  fields = _messages.StringField(5)
  key = _messages.StringField(6)
  oauth_token = _messages.StringField(7)
  prettyPrint = _messages.BooleanField(8, default=True)
  quotaUser = _messages.StringField(9)
  trace = _messages.StringField(10)
  uploadType = _messages.StringField(11)
  upload_protocol = _messages.StringField(12)


class StringContent(_messages.Message):
  r"""String content for a constraint resource.

  Fields:
    yaml: string json = 2;
  """

  yaml = _messages.StringField(1)


class Target(_messages.Message):
  r"""Target defines which resources this template targets.

  Fields:
    regoPolicy: Reserved: The policy defined using CEL. CELPolicy cel_policy =
      3;
    target: spec.targets.target.
  """

  regoPolicy = _messages.MessageField('RegoPolicy', 1)
  target = _messages.StringField(2)


encoding.AddCustomJsonFieldMapping(
    StandardQueryParameters, 'f__xgafv', '$.xgafv')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_1', '1')
encoding.AddCustomJsonEnumMapping(
    StandardQueryParameters.FXgafvValueValuesEnum, '_2', '2')



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""API utilities for `gcloud network-services` commands."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.calliope import base
from googlecloudsdk.core import resources

API_VERSION_FOR_TRACK = {
    base.ReleaseTrack.ALPHA: 'v1alpha1',
    base.ReleaseTrack.BETA: 'v1beta1',
    base.ReleaseTrack.GA: 'v1',
}
API_NAME = 'networkservices'


def GetMessagesModule(release_track=base.ReleaseTrack.BETA):
  api_version = API_VERSION_FOR_TRACK.get(release_track)
  return apis.GetMessagesModule(API_NAME, api_version)


def GetClientInstance(release_track=base.ReleaseTrack.BETA):
  api_version = API_VERSION_FOR_TRACK.get(release_track)
  return apis.GetClientInstance(API_NAME, api_version)


def GetApiBaseUrl(release_track=base.ReleaseTrack.ALPHA):
  api_version = API_VERSION_FOR_TRACK.get(release_track)
  return resources.GetApiBaseUrlOrThrow(API_NAME, api_version)



# -*- coding: utf-8 -*- #
# Copyright 2014 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for creating target TCP proxies."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import target_proxies_utils
from googlecloudsdk.api_lib.compute import utils
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import scope as compute_scope
from googlecloudsdk.command_lib.compute.backend_services import (
    flags as backend_service_flags)
from googlecloudsdk.command_lib.compute.target_tcp_proxies import flags


@base.ReleaseTracks(base.ReleaseTrack.GA)
class Create(base.CreateCommand):
  """Create a target TCP proxy."""

  BACKEND_SERVICE_ARG = None
  TARGET_TCP_PROXY_ARG = None

  _enable_region_target_tcp_proxy = True

  @classmethod
  def Args(cls, parser):
    target_proxies_utils.AddProxyHeaderRelatedCreateArgs(parser)

    cls.BACKEND_SERVICE_ARG = (
        backend_service_flags.BackendServiceArgumentForTargetTcpProxy(
            allow_regional=cls._enable_region_target_tcp_proxy))
    cls.BACKEND_SERVICE_ARG.AddArgument(parser)
    cls.TARGET_TCP_PROXY_ARG = flags.TargetTcpProxyArgument(
        allow_regional=cls._enable_region_target_tcp_proxy)
    cls.TARGET_TCP_PROXY_ARG.AddArgument(parser, operation_type='create')

    flags.AddProxyBind(parser)

    parser.add_argument(
        '--description',
        help='An optional, textual description for the target TCP proxy.')

    if cls._enable_region_target_tcp_proxy:
      parser.display_info.AddCacheUpdater(flags.TargetTcpProxiesCompleter)
    else:
      parser.display_info.AddCacheUpdater(flags.GATargetTcpProxiesCompleter)

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())

    if self._enable_region_target_tcp_proxy:
      if not (args.backend_service_region or args.global_backend_service):
        # infer scope from proxy if not explicitly specified
        args.backend_service_region = getattr(args, 'region', None)
        args.global_backend_service = getattr(args, 'global', None)

    backend_service_ref = self.BACKEND_SERVICE_ARG.ResolveAsResource(
        args, holder.resources, default_scope=compute_scope.ScopeEnum.GLOBAL)

    target_tcp_proxy_ref = self.TARGET_TCP_PROXY_ARG.ResolveAsResource(
        args, holder.resources, default_scope=compute_scope.ScopeEnum.GLOBAL)

    messages = holder.client.messages
    if args.proxy_header:
      proxy_header = messages.TargetTcpProxy.ProxyHeaderValueValuesEnum(
          args.proxy_header)
    else:
      proxy_header = (messages.TargetTcpProxy.ProxyHeaderValueValuesEnum.NONE)

    target_tcp_proxy = messages.TargetTcpProxy(
        description=args.description,
        name=target_tcp_proxy_ref.Name(),
        proxyHeader=proxy_header,
        service=backend_service_ref.SelfLink())

    if args.proxy_bind is not None:
      target_tcp_proxy.proxyBind = args.proxy_bind

    return self._MakeRequest(target_tcp_proxy_ref, target_tcp_proxy, holder)

  def _MakeRequest(self, target_tcp_proxy_ref, target_tcp_proxy, holder):
    if target_tcp_proxy_ref.Collection() == 'compute.regionTargetTcpProxies':
      return self._MakeRegionalRequest(target_tcp_proxy_ref, target_tcp_proxy,
                                       holder)
    return self._MakeGlobalRequest(target_tcp_proxy_ref, target_tcp_proxy,
                                   holder)

  def _MakeRegionalRequest(self, target_tcp_proxy_ref, target_tcp_proxy,
                           holder):
    client = holder.client.apitools_client
    messages = holder.client.messages
    request = messages.ComputeRegionTargetTcpProxiesInsertRequest(
        project=target_tcp_proxy_ref.project,
        targetTcpProxy=target_tcp_proxy,
        region=target_tcp_proxy_ref.region)

    errors = []
    resources = holder.client.MakeRequests(
        [(client.regionTargetTcpProxies, 'Insert', request)], errors)

    if errors:
      utils.RaiseToolException(errors)
    return resources

  def _MakeGlobalRequest(self, target_tcp_proxy_ref, target_tcp_proxy, holder):
    client = holder.client.apitools_client
    messages = holder.client.messages
    request = messages.ComputeTargetTcpProxiesInsertRequest(
        project=target_tcp_proxy_ref.project, targetTcpProxy=target_tcp_proxy)

    errors = []
    resources = holder.client.MakeRequests(
        [(client.targetTcpProxies, 'Insert', request)], errors)

    if errors:
      utils.RaiseToolException(errors)
    return resources


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA)
class CreateAlphaBeta(Create):
  _enable_region_target_tcp_proxy = True


Create.detailed_help = {
    'brief':
        'Create a target TCP proxy',
    'DESCRIPTION':
        """
        *{command}* is used to create target TCP proxies. A target
        TCP proxy is referenced by one or more forwarding rules which
        define which packets the proxy is responsible for routing. The
        target TCP proxy points to a backend service which handle the
        actual requests.
        """,
}



# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""types create command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.deployment_manager import dm_base
from googlecloudsdk.api_lib.deployment_manager import dm_labels
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.deployment_manager import composite_types
from googlecloudsdk.command_lib.deployment_manager import dm_util
from googlecloudsdk.command_lib.deployment_manager import dm_write
from googlecloudsdk.command_lib.deployment_manager import flags
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.core import log


def LogResource(request, is_async):
  log.CreatedResource(request.compositeType.name,
                      kind='composite_type',
                      is_async=is_async)


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA)
@dm_base.UseDmApi(dm_base.DmApiVersion.V2BETA)
class Create(base.CreateCommand, dm_base.DmCommand):
  """Create a type.

  This command inserts (creates) a new composite type based on a provided
  configuration file.
  """

  detailed_help = {
      'EXAMPLES': """
To create a new composite type, run:

  $ {command} my-composite-type --template=my-template.jinja --status=EXPERIMENTAL --description="My type."
""",
  }

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command.

    Args:
      parser: An argparse parser that you can use to add arguments that go
          on the command line after this command. Positional arguments are
          allowed.
    """
    flags.AddAsyncFlag(parser)
    composite_types.AddCompositeTypeNameFlag(parser)
    composite_types.AddTemplateFlag(parser)
    composite_types.AddDescriptionFlag(parser)
    composite_types.AddStatusFlag(parser)
    labels_util.AddCreateLabelsFlags(parser)

  def Run(self, args):
    """Run 'types create'.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
          with.

    Raises:
      HttpException: An http error response was received while executing api
          request.
    """
    composite_type_ref = composite_types.GetReference(self.resources, args.name)
    update_labels_dict = labels_util.GetUpdateLabelsDictFromArgs(args)
    labels = dm_labels.UpdateLabels([],
                                    self.messages.CompositeTypeLabelEntry,
                                    update_labels=update_labels_dict)
    template_contents = composite_types.TemplateContentsFor(self.messages,
                                                            args.template)

    computed_status = self.messages.CompositeType.StatusValueValuesEnum(
        args.status) if args.status is not None else None

    composite_type = self.messages.CompositeType(
        name=args.name,
        description=args.description,
        status=computed_status,
        templateContents=template_contents,
        labels=labels)
    request = self.messages.DeploymentmanagerCompositeTypesInsertRequest(
        project=composite_type_ref.project,
        compositeType=composite_type)

    response = dm_write.Execute(self.client, self.messages, self.resources,
                                request, args.async_,
                                self.client.compositeTypes.Insert, LogResource)
    dm_util.LogOperationStatus(response, 'Create')



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Access approval requests API helper."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.py import list_pager
from googlecloudsdk.api_lib.util import apis


def Approve(name):
  """Approve an approval request."""
  client = apis.GetClientInstance('accessapproval', 'v1')
  msgs = apis.GetMessagesModule('accessapproval', 'v1')

  if 'organizations/' in name:
    req = msgs.AccessapprovalOrganizationsApprovalRequestsApproveRequest(
        name=name)
    return client.organizations_approvalRequests.Approve(req)
  if 'folders/' in name:
    req = msgs.AccessapprovalFoldersApprovalRequestsApproveRequest(name=name)
    return client.folders_approvalRequests.Approve(req)

  req = msgs.AccessapprovalProjectsApprovalRequestsApproveRequest(name=name)
  return client.projects_approvalRequests.Approve(req)


def Dismiss(name):
  """Dismiss an approval request."""
  client = apis.GetClientInstance('accessapproval', 'v1')
  msgs = apis.GetMessagesModule('accessapproval', 'v1')

  if 'organizations/' in name:
    req = msgs.AccessapprovalOrganizationsApprovalRequestsDismissRequest(
        name=name)
    return client.organizations_approvalRequests.Dismiss(req)
  if 'folders/' in name:
    req = msgs.AccessapprovalFoldersApprovalRequestsDismissRequest(name=name)
    return client.folders_approvalRequests.Dismiss(req)

  req = msgs.AccessapprovalProjectsApprovalRequestsDismissRequest(name=name)
  return client.projects_approvalRequests.Dismiss(req)


def Invalidate(name):
  """Invalidate an approval request."""
  client = apis.GetClientInstance('accessapproval', 'v1')
  msgs = apis.GetMessagesModule('accessapproval', 'v1')

  if 'organizations/' in name:
    req = msgs.AccessapprovalOrganizationsApprovalRequestsInvalidateRequest(
        name=name)
    return client.organizations_approvalRequests.Invalidate(req)
  if 'folders/' in name:
    req = msgs.AccessapprovalFoldersApprovalRequestsInvalidateRequest(name=name)
    return client.folders_approvalRequests.Invalidate(req)

  req = msgs.AccessapprovalProjectsApprovalRequestsInvalidateRequest(name=name)
  return client.projects_approvalRequests.Invalidate(req)


def Get(name):
  """Get an approval request by name."""
  client = apis.GetClientInstance('accessapproval', 'v1')
  msgs = apis.GetMessagesModule('accessapproval', 'v1')

  if 'organizations/' in name:
    req = msgs.AccessapprovalOrganizationsApprovalRequestsGetRequest(name=name)
    return client.organizations_approvalRequests.Get(req)
  if 'folders/' in name:
    req = msgs.AccessapprovalFoldersApprovalRequestsGetRequest(name=name)
    return client.folders_approvalRequests.Get(req)

  req = msgs.AccessapprovalProjectsApprovalRequestsGetRequest(name=name)
  return client.projects_approvalRequests.Get(req)


def List(parent, filter=None):
  """List approval requests for the parent resource."""
  client = apis.GetClientInstance('accessapproval', 'v1')
  msgs = apis.GetMessagesModule('accessapproval', 'v1')
  req = None
  svc = None

  if 'organizations/' in parent:
    req = msgs.AccessapprovalOrganizationsApprovalRequestsListRequest(
        parent=parent)
    svc = client.organizations_approvalRequests
  elif 'folders/' in parent:
    req = msgs.AccessapprovalFoldersApprovalRequestsListRequest(parent=parent)
    svc = client.folders_approvalRequests
  else:
    req = msgs.AccessapprovalProjectsApprovalRequestsListRequest(parent=parent)
    svc = client.projects_approvalRequests

  if filter:
    req.filter = filter
  else:
    req.filter = 'PENDING'

  return list_pager.YieldFromList(
      svc, req, field='approvalRequests', batch_size_attribute='pageSize')



# Copyright 2017 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Integration helpers.

This module provides helpers for integrating with `requests-oauthlib`_.
Typically, you'll want to use the higher-level helpers in
:mod:`google_auth_oauthlib.flow`.

.. _requests-oauthlib: http://requests-oauthlib.readthedocs.io/en/stable/
"""

import datetime
import json

from google.auth import external_account_authorized_user
import google.oauth2.credentials
import requests_oauthlib

_REQUIRED_CONFIG_KEYS = frozenset(("auth_uri", "token_uri", "client_id"))


def session_from_client_config(client_config, scopes, **kwargs):
    """Creates a :class:`requests_oauthlib.OAuth2Session` from client
    configuration loaded from a Google-format client secrets file.

    Args:
        client_config (Mapping[str, Any]): The client
            configuration in the Google `client secrets`_ format.
        scopes (Sequence[str]): The list of scopes to request during the
            flow.
        kwargs: Any additional parameters passed to
            :class:`requests_oauthlib.OAuth2Session`

    Raises:
        ValueError: If the client configuration is not in the correct
            format.

    Returns:
        Tuple[requests_oauthlib.OAuth2Session, Mapping[str, Any]]: The new
            oauthlib session and the validated client configuration.

    .. _client secrets:
        https://github.com/googleapis/google-api-python-client/blob/main/docs/client-secrets.md
    """

    if "web" in client_config:
        config = client_config["web"]
    elif "installed" in client_config:
        config = client_config["installed"]
    else:
        raise ValueError("Client secrets must be for a web or installed app.")

    if not _REQUIRED_CONFIG_KEYS.issubset(config.keys()):
        raise ValueError("Client secrets is not in the correct format.")

    session = requests_oauthlib.OAuth2Session(
        client_id=config["client_id"], scope=scopes, **kwargs
    )

    return session, client_config


def session_from_client_secrets_file(client_secrets_file, scopes, **kwargs):
    """Creates a :class:`requests_oauthlib.OAuth2Session` instance from a
    Google-format client secrets file.

    Args:
        client_secrets_file (str): The path to the `client secrets`_ .json
            file.
        scopes (Sequence[str]): The list of scopes to request during the
            flow.
        kwargs: Any additional parameters passed to
            :class:`requests_oauthlib.OAuth2Session`

    Returns:
        Tuple[requests_oauthlib.OAuth2Session, Mapping[str, Any]]: The new
            oauthlib session and the validated client configuration.

    .. _client secrets:
        https://github.com/googleapis/google-api-python-client/blob/main/docs/client-secrets.md
    """
    with open(client_secrets_file, "r") as json_file:
        client_config = json.load(json_file)

    return session_from_client_config(client_config, scopes, **kwargs)


def credentials_from_session(session, client_config=None):
    """Creates :class:`google.oauth2.credentials.Credentials` from a
    :class:`requests_oauthlib.OAuth2Session`.

    :meth:`fetch_token` must be called on the session before before calling
    this. This uses the session's auth token and the provided client
    configuration to create :class:`google.oauth2.credentials.Credentials`.
    This allows you to use the credentials from the session with Google
    API client libraries.

    Args:
        session (requests_oauthlib.OAuth2Session): The OAuth 2.0 session.
        client_config (Mapping[str, Any]): The subset of the client
            configuration to use. For example, if you have a web client
            you would pass in `client_config['web']`.

    Returns:
        google.oauth2.credentials.Credentials: The constructed credentials.

    Raises:
        ValueError: If there is no access token in the session.
    """
    client_config = client_config if client_config is not None else {}

    if not session.token:
        raise ValueError(
            "There is no access token for this session, did you call " "fetch_token?"
        )

    if "3pi" in client_config:
        credentials = external_account_authorized_user.Credentials(
            token=session.token["access_token"],
            refresh_token=session.token.get("refresh_token"),
            token_url=client_config.get("token_uri"),
            client_id=client_config.get("client_id"),
            client_secret=client_config.get("client_secret"),
            token_info_url=client_config.get("token_info_url"),
            scopes=session.scope,
        )
    else:
        credentials = google.oauth2.credentials.Credentials(
            session.token["access_token"],
            refresh_token=session.token.get("refresh_token"),
            id_token=session.token.get("id_token"),
            token_uri=client_config.get("token_uri"),
            client_id=client_config.get("client_id"),
            client_secret=client_config.get("client_secret"),
            scopes=session.scope,
        )
    credentials.expiry = datetime.datetime.utcfromtimestamp(session.token["expires_at"])
    return credentials



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Update policy command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.container.binauthz import apis
from googlecloudsdk.api_lib.container.binauthz import platform_policy
from googlecloudsdk.api_lib.util import messages as messages_util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.container.binauthz import flags
from googlecloudsdk.command_lib.container.binauthz import parsing


@base.Hidden
@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA)
class Update(base.UpdateCommand):
  # pylint: disable=line-too-long
  """Update a policy.

  ## EXAMPLES

  To update an existing policy using its resource name:

    $ {command} projects/my_proj/platforms/gke/policies/policy1 --policy-file=policy1.json

  To update the same policy using flags:

    $ {command} policy1 --platform=gke --project=my_proj --policy-file=policy1.json
  """
  # pylint: enable=line-too-long

  @staticmethod
  def Args(parser):
    flags.AddPlatformPolicyResourceArg(parser, 'to update')
    parser.add_argument(
        '--policy-file',
        required=True,
        help='The JSON or YAML file containing the new policy.')
    parser.display_info.AddFormat('yaml')

  def Run(self, args):
    # The API is only available in v1.
    messages = apis.GetMessagesModule('v1')
    policy_ref = args.CONCEPTS.policy_resource_name.Parse().RelativeName()
    # Load the policy file into a Python dict.
    policy_obj = parsing.LoadResourceFile(args.policy_file)
    # Decode the dict into a PlatformPolicy message, allowing DecodeErrors to
    # bubble up to the user if they are raised.
    policy = messages_util.DictToMessageWithErrorCheck(policy_obj,
                                                       messages.PlatformPolicy)
    return platform_policy.Client('v1').Update(policy_ref, policy)



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Common utility functions to consturct compute future reservations message."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.compute.reservations import util as reservation_util


def MakeFutureReservationMessageFromArgs(messages, resources, args,
                                         future_reservation_ref):
  """Construct future reservation message from args passed in."""
  local_ssds = reservation_util.MakeLocalSsds(messages,
                                              getattr(args, 'local_ssd', None))
  accelerators = reservation_util.MakeGuestAccelerators(
      messages, getattr(args, 'accelerator', None))
  allocated_instance_properties = MakeAllocatedInstanceProperties(
      messages, args.machine_type, args.min_cpu_platform, local_ssds,
      accelerators, getattr(args, 'location_hint', None),
      getattr(args, 'maintenance_freeze_duration', None),
      getattr(args, 'maintenance_interval', None))
  sku_properties = MakeSpecificSKUPropertiesMessage(
      messages, resources, allocated_instance_properties, args.total_count,
      getattr(args, 'source_instance_template', None))
  time_window = MakeTimeWindowMessage(messages, args.start_time,
                                      getattr(args, 'end_time', None),
                                      getattr(args, 'duration', None))
  share_settings = MakeShareSettings(messages, args,
                                     getattr(args, 'share_setting', None))
  planning_status = MakePlanningStatus(messages,
                                       getattr(args, 'planning_status', None))
  return MakeFutureReservationMessage(messages, future_reservation_ref.Name(),
                                      sku_properties, time_window,
                                      share_settings,
                                      planning_status)


def MakeAllocatedInstanceProperties(messages,
                                    machine_type,
                                    min_cpu_platform,
                                    local_ssds,
                                    accelerators,
                                    location_hint=None,
                                    freeze_duration=None,
                                    freeze_interval=None):
  """Constructs an instance propteries for reservations message object."""
  prop_msgs = (
      messages.AllocationSpecificSKUAllocationReservedInstanceProperties)
  instance_properties = prop_msgs(
      machineType=machine_type,
      guestAccelerators=accelerators,
      minCpuPlatform=min_cpu_platform,
      localSsds=local_ssds)
  if location_hint:
    instance_properties.locationHint = location_hint
  if freeze_duration:
    instance_properties.maintenanceFreezeDurationHours = freeze_duration // 3600
  if freeze_interval:
    instance_properties.maintenanceInterval = (
        messages.AllocationSpecificSKUAllocationReservedInstanceProperties
        .MaintenanceIntervalValueValuesEnum(freeze_interval))
  return instance_properties


def MakeSpecificSKUPropertiesMessage(messages,
                                     resources,
                                     instance_properties,
                                     total_count,
                                     source_instance_template=None):
  """Constructs a specific sku properties message object."""
  properties = None
  source_instance_template_url = None
  if source_instance_template:
    source_instance_template_url = reservation_util.MakeInstanceTemplateUrl(
        source_instance_template, resources)
  else:
    properties = instance_properties
  return messages.FutureReservationSpecificSKUProperties(
      totalCount=total_count,
      sourceInstanceTemplate=source_instance_template_url,
      instanceProperties=properties)


def MakeTimeWindowMessage(messages, start_time, end_time, duration):
  """Constructs the time window message object."""
  if end_time:
    return messages.FutureReservationTimeWindow(
        startTime=start_time, endTime=end_time)
  else:
    return messages.FutureReservationTimeWindow(
        startTime=start_time, duration=messages.Duration(seconds=duration))


def MakeShareSettings(messages, args, setting_configs):
  """Constructs the share settings message object."""
  if setting_configs:
    if setting_configs == 'local':
      if args.IsSpecified('share_with'):
        raise exceptions.InvalidArgumentException(
            '--share_with',
            'The scope this reservation is to be shared with must not be '
            'specified with share setting local.')
      return messages.ShareSettings(shareType=messages.ShareSettings
                                    .ShareTypeValueValuesEnum.LOCAL)
    if setting_configs == 'projects':
      if not args.IsSpecified('share_with'):
        raise exceptions.InvalidArgumentException(
            '--share_with',
            'The projects this reservation is to be shared with must be '
            'specified.')
      return messages.ShareSettings(
          shareType=messages.ShareSettings.ShareTypeValueValuesEnum
          .SPECIFIC_PROJECTS,
          projects=getattr(args, 'share_with', None))
  else:
    return None


def MakePlanningStatus(messages, planning_status):
  """Constructs the planning status enum value."""
  if planning_status:
    if planning_status == 'SUBMITTED':
      return messages.FutureReservation.PlanningStatusValueValuesEnum.SUBMITTED
  return None


def MakeFutureReservationMessage(messages, reservation_name, sku_properties,
                                 time_window, share_settings, planning_status):
  """Constructs a future reservation message object."""
  future_reservation_message = messages.FutureReservation(
      name=reservation_name,
      specificSkuProperties=sku_properties,
      timeWindow=time_window,
      planningStatus=planning_status)
  if share_settings:
    future_reservation_message.shareSettings = share_settings
  return future_reservation_message



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Set up flags for creating triggers."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.cloudbuild import cloudbuild_util
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import exceptions as c_exceptions
from googlecloudsdk.command_lib.builds import flags as build_flags

_CREATE_FILE_DESC = ('A file that contains the configuration for the '
                     'WorkerPool to be created.')
_UPDATE_FILE_DESC = ('A file that contains updates to the configuration for '
                     'the WorkerPool.')


def AddTriggerArgs(parser, add_region_flag=True, add_name=True):
  """Set up the generic argparse flags for creating or updating a build trigger.

  Args:
    parser: An argparse.ArgumentParser-like object.
    add_region_flag: If true, the default region flag is added.
    add_name: If true, the trigger name is added.

  Returns:
    An empty parser group to be populated with flags specific to a trigger-type.
  """

  parser.display_info.AddFormat("""
          table(
            name,
            createTime.date('%Y-%m-%dT%H:%M:%S%Oz', undefined='-'),
            status
          )
        """)

  trigger_config = parser.add_mutually_exclusive_group(required=True)

  AddTriggerConfigFilePathArg(trigger_config)

  # Trigger configuration
  flag_config = trigger_config.add_argument_group(
      help='Flag based trigger configuration')
  if add_region_flag:
    build_flags.AddRegionFlag(flag_config, hidden=False, required=False)
  AddFlagConfigArgs(flag_config, add_name)

  return flag_config


def AddGitLabEnterpriseTriggerArgs(parser):
  """Set up the generic argparse flags for creating or updating a build trigger for GitLab Enterprise.

  Args:
    parser: An argparse.ArgumentParser-like object.

  Returns:
    An empty parser group to be populated with flags specific to a trigger-type.
  """

  parser.display_info.AddFormat("""
          table(
            name,
            createTime.date('%Y-%m-%dT%H:%M:%S%Oz', undefined='-'),
            status
          )
        """)

  trigger_config = parser.add_mutually_exclusive_group(required=True)

  AddTriggerConfigFilePathArg(trigger_config)

  # Trigger configuration
  flag_config = trigger_config.add_argument_group(
      help='Flag based trigger configuration')
  build_flags.AddRegionFlag(flag_config, hidden=False, required=False)
  AddFlagConfigArgs(flag_config)

  return flag_config


def AddFlagConfigArgs(flag_config, add_name=True):
  """Adds additional argparse flags related to flag config.

  Args:
    flag_config: argparse argument group. Additional flags will be added to this
      group to cover common flag configuration settings.
    add_name: If true, the trigger name is added.
  """

  if add_name:
    flag_config.add_argument('--name', help='Build trigger name.')
  flag_config.add_argument('--description', help='Build trigger description.')
  flag_config.add_argument(
      '--service-account',
      help=(
          'The service account used for all user-controlled operations '
          'including UpdateBuildTrigger, RunBuildTrigger, CreateBuild, and '
          'CancelBuild. If no service account is set, then the standard Cloud '
          'Build service account ([PROJECT_NUM]@system.gserviceaccount.com) is '
          'used instead. Format: '
          '`projects/{PROJECT_ID}/serviceAccounts/{ACCOUNT_ID_OR_EMAIL}`.'),
      required=False)
  flag_config.add_argument(
      '--require-approval',
      help='Require manual approval for triggered builds.',
      action='store_true')


def AddTriggerConfigFilePathArg(trigger_config):
  """Allow trigger config to be specified on the command line or STDIN.

  Args:
    trigger_config: the config of which the file path can be specified.
  """

  trigger_config.add_argument(
      '--trigger-config',
      help=(
          'Path to Build Trigger config file (JSON or YAML format). For more '
          'details, see https://cloud.google.com/cloud-build/docs/api/'
          'reference/rest/v1/projects.triggers#BuildTrigger'
      ),
      metavar='PATH',
  )


def ParseTriggerArgs(args, messages):
  """Parses flags generic to all triggers.

  Args:
    args: An argparse arguments object.
    messages: A Cloud Build messages module

  Returns:
    A partially populated build trigger and a boolean indicating whether or not
    the full trigger was loaded from a file.
  """
  if args.trigger_config:
    trigger = cloudbuild_util.LoadMessageFromPath(
        path=args.trigger_config,
        msg_type=messages.BuildTrigger,
        msg_friendly_name='build trigger config',
        skip_camel_case=['substitutions'])
    return trigger, True

  trigger = messages.BuildTrigger()
  trigger.name = args.name
  trigger.description = args.description
  trigger.serviceAccount = args.service_account
  ParseRequireApproval(trigger, args, messages)
  return trigger, False


def ParseTriggerArgsForUpdate(trigger_args, messages):
  """Parses flags generic to all triggers.

  Args:
    trigger_args: An argparse arguments object.
    messages: A Cloud Build messages module

  Returns:
    A partially populated build trigger and a boolean indicating whether or not
    the full trigger was loaded from a file.
  """
  if trigger_args.trigger_config:
    trigger = cloudbuild_util.LoadMessageFromPath(
        path=trigger_args.trigger_config,
        msg_type=messages.BuildTrigger,
        msg_friendly_name='build trigger config',
        skip_camel_case=['substitutions'])
    return trigger, True

  trigger = messages.BuildTrigger()
  trigger.description = trigger_args.description
  trigger.serviceAccount = trigger_args.service_account
  ParseRequireApproval(trigger, trigger_args, messages)
  return trigger, False


def AddIncludeLogsArgs(flag_config):
  """Add flag related to including logs for GitHub checkrun summary page.

  Args:
    flag_config: argparse argument group. Include logs for GitHub will be
    added to this config.
  """

  flag_config.add_argument(
      '--include-logs-with-status',
      help=(
          'Build logs will be sent back to GitHub as part of the checkrun '
          'result.'
      ), action='store_true')


def AddRepoEventArgs(flag_config):
  """Adds additional argparse flags related to repo events.

  Args:
    flag_config: argparse argument group. Additional flags will be added to this
      group to cover common build configuration settings.
  """

  flag_config.add_argument(
      '--included-files',
      help=(
          'Glob filter. Changes affecting at least one included file will'
          ' trigger builds.'
      ),
      type=arg_parsers.ArgList(),
      metavar='GLOB',
  )
  flag_config.add_argument(
      '--ignored-files',
      help=("""\
 Glob filter. Changes only affecting ignored files won't trigger builds.
"""),
      type=arg_parsers.ArgList(),
      metavar='GLOB',
  )


def AddFilterArg(flag_config):
  """Adds trigger filter flag arg.

  Args:
    flag_config: argparse argument group. Trigger filter flag will be added to
      this config.
  """
  flag_config.add_argument(
      '--subscription-filter',
      help="""\
CEL filter expression for the trigger.
See https://cloud.google.com/build/docs/filter-build-events-using-cel for more details.
""")


def AddSubstitutions(argument_group):
  """Adds a substituion flag to the given argument group.

  Args:
    argument_group: argparse argument group to which the substitution flag will
      be added.
  """

  argument_group.add_argument(
      '--substitutions',
      metavar='KEY=VALUE',
      type=arg_parsers.ArgDict(),
      help="""\
Parameters to be substituted in the build specification.

For example (using some nonsensical substitution keys; all keys must begin with
an underscore):

  $ gcloud builds triggers create ... --config config.yaml
      --substitutions _FAVORITE_COLOR=blue,_NUM_CANDIES=10

This will result in a build where every occurrence of ```${_FAVORITE_COLOR}```
in certain fields is replaced by "blue", and similarly for ```${_NUM_CANDIES}```
and "10".

Only the following built-in variables can be specified with the
`--substitutions` flag: REPO_NAME, BRANCH_NAME, TAG_NAME, REVISION_ID,
COMMIT_SHA, SHORT_SHA.

For more details, see:
https://cloud.google.com/cloud-build/docs/api/build-requests#substitutions
""")


def AddSubstitutionUpdatingFlags(argument_group):
  """Adds substitution updating flags to the given argument group.

  Args:
    argument_group: argparse argument group to which the substitution updating
      flags flag will be added.
  """

  argument_group.add_argument(
      '--update-substitutions',
      metavar='KEY=VALUE',
      type=arg_parsers.ArgDict(),
      help="""\
Update or add to existing substitutions.
Substitutions are parameters to be substituted or add in the build specification.

For example (using some nonsensical substitution keys; all keys must begin with
an underscore):

  $ gcloud builds triggers update ...
      --update-substitutions _FAVORITE_COLOR=blue,_NUM_CANDIES=10

This will add the provided substitutions to the existing substitutions and
results in a build where every occurrence of ```${_FAVORITE_COLOR}```
in certain fields is replaced by "blue", and similarly for ```${_NUM_CANDIES}```
and "10".

Only the following built-in variables can be specified with the
`--substitutions` flag: REPO_NAME, BRANCH_NAME, TAG_NAME, REVISION_ID,
COMMIT_SHA, SHORT_SHA.

For more details, see:
https://cloud.google.com/build/docs/build-config-file-schema#substitutions
""")

  argument_group.add_argument(
      '--clear-substitutions',
      action='store_true',
      help='Clear existing substitutions.')

  argument_group.add_argument(
      '--remove-substitutions',
      metavar='KEY',
      type=arg_parsers.ArgList(),
      help='Remove existing substitutions if present.')


def AddBuildConfigArgs(flag_config, add_docker_args=True):
  """Adds additional argparse flags to a group for build configuration options.

  Args:
    flag_config: argparse argument group. Additional flags will be added to this
      group to cover common build configuration settings.
    add_docker_args: If true, docker args are added automatically.
  Returns:
    build_config: a build config.
  """

  # Build config and inline config support substitutions whereas dockerfile
  # does not. We can't have a flag with the same name in two separate
  # groups so we have to have one flag outside of the config argument group.
  AddSubstitutions(flag_config)

  build_config = flag_config.add_mutually_exclusive_group(required=True)

  build_config.add_argument(
      '--build-config',
      metavar='PATH',
      help="""\
Path to a YAML or JSON file containing the build configuration in the repository.

For more details, see: https://cloud.google.com/cloud-build/docs/build-config
""")

  build_config.add_argument(
      '--inline-config',
      metavar='PATH',
      help="""\
      Local path to a YAML or JSON file containing a build configuration.
    """)

  if add_docker_args:
    AddBuildDockerArgs(build_config)
  return build_config


def AddBuildDockerArgs(argument_group, require_docker_image=False):
  """Adds additional argparse flags to a group for build docker options.

  Args:
    argument_group: argparse argument group to which build docker flag will
      be added.
    require_docker_image: If true, --dockerfile-image must be provided.
  """
  docker = argument_group.add_argument_group(
      help='Dockerfile build configuration flags')
  docker.add_argument(
      '--dockerfile',
      required=True,
      help="""\
Path of Dockerfile to use for builds in the repository.

If specified, a build config will be generated to run docker
build using the specified file.

The filename is relative to the Dockerfile directory.
""")
  docker.add_argument(
      '--dockerfile-dir',
      default='/',
      help="""\
Location of the directory containing the Dockerfile in the repository.

The directory will also be used as the Docker build context.
""")

  docker_image_help_text = """\
Docker image name to build.

If not specified, gcr.io/PROJECT/github.com/REPO_OWNER/REPO_NAME:$COMMIT_SHA will be used.

Use a build configuration (cloudbuild.yaml) file for building multiple images in a single trigger.
"""
  if require_docker_image:
    docker_image_help_text = 'Docker image name to build.'
  docker.add_argument(
      '--dockerfile-image',
      required=require_docker_image,
      help=docker_image_help_text)


def AddBuildConfigArgsForUpdate(flag_config,
                                has_build_config=False):
  """Adds additional argparse flags to a group for build configuration options for update command.

  Args:
    flag_config: Argparse argument group. Additional flags will be added to this
      group to cover common build configuration settings.
    has_build_config: Whether it is possible for the trigger to have
      filename.

  Returns:
    build_config: A build config.
  """

  # Build config and inline config support substitutions whereas Dockerfile
  # does not. Cloud Build cannot have a flag with the same name in two separate
  # groups. You must add a flag outside of the config argument group.
  substitutions = flag_config.add_argument_group()
  AddSubstitutionUpdatingFlags(substitutions)

  build_config = flag_config.add_mutually_exclusive_group()

  if has_build_config:
    build_config.add_argument(
        '--build-config',
        metavar='PATH',
        help="""\
  Path to a YAML or JSON file containing the build configuration in the repository.

  For more details, see: https://cloud.google.com/cloud-build/docs/build-config
  """)

  build_config.add_argument(
      '--inline-config',
      metavar='PATH',
      help="""\
      Local path to a YAML or JSON file containing a build configuration.
    """)

  AddBuildDockerArgs(build_config, require_docker_image=True)

  return build_config


def ParseRepoEventArgs(trigger, args):
  """Parses repo event related flags.

  Args:
    trigger: The trigger to populate.
    args: An argparse arguments object.
  """
  if args.included_files:
    trigger.includedFiles = args.included_files
  if args.ignored_files:
    trigger.ignoredFiles = args.ignored_files


def ParseBuildConfigArgs(trigger,
                         args,
                         messages,
                         default_image,
                         need_repo=False):
  """Parses build-config flags.

  Args:
    trigger: The trigger to populate.
    args: An argparse arguments object.
    messages: A Cloud Build messages module.
    default_image: The docker image to use if args.dockerfile_image is empty.
    need_repo: Whether or not a repo needs to be included explicitly in flags.
  """
  if args.build_config:
    # If we don't need a repo, then the repository information is already known
    # and we just need the filename. Otherwise, this trigger needs to
    # be a GitFileSource trigger (which is taken care of in ParseGitRepoSource).
    if not need_repo:
      trigger.filename = args.build_config
    trigger.substitutions = cloudbuild_util.EncodeTriggerSubstitutions(
        args.substitutions, messages)
  if args.dockerfile:

    if args.substitutions:
      raise c_exceptions.ConflictingArgumentsException(
          'Dockerfile and substitutions',
          'Substitutions are not supported with a Dockerfile configuration.')

    image = args.dockerfile_image or default_image
    trigger.build = messages.Build(steps=[
        messages.BuildStep(
            name='gcr.io/cloud-builders/docker',
            dir=args.dockerfile_dir,
            args=['build', '-t', image, '-f', args.dockerfile, '.'],
        )
    ])
  if args.inline_config:
    trigger.build = cloudbuild_util.LoadMessageFromPath(args.inline_config,
                                                        messages.Build,
                                                        'inline build config',
                                                        ['substitutions'])
    trigger.substitutions = cloudbuild_util.EncodeTriggerSubstitutions(
        args.substitutions, messages)

  if need_repo:
    # Repo is required if a build config (filename) or dockerfile was provided.
    required = args.build_config or args.dockerfile
    ParseGitRepoSource(trigger, args, messages, required=required)


def ParseBuildConfigArgsForUpdate(trigger,
                                  old_trigger,
                                  args,
                                  messages,
                                  has_build_config=False):
  """Parses build-config flags for update command.

  Args:
    trigger: The trigger to populate.
    old_trigger: The existing trigger to be updated.
    args: An argparse arguments object.
    messages: A Cloud Build messages module.
    has_build_config: Whether it is possible for the trigger to have
      filename.
  """
  if has_build_config:
    trigger.filename = args.build_config

  if args.dockerfile:
    if args.update_substitutions:
      raise c_exceptions.ConflictingArgumentsException(
          'Dockerfile and substitutions',
          'Substitutions are not supported with a Dockerfile configuration.')

    if args.dockerfile_dir:
      dockerfile_dir = args.dockerfile_dir
    elif old_trigger.build and old_trigger.build.steps:
      dockerfile_dir = old_trigger.steps[0].dir
    else:
      dockerfile_dir = '/'

    trigger.build = messages.Build(steps=[
        messages.BuildStep(
            name='gcr.io/cloud-builders/docker',
            dir=dockerfile_dir,
            args=[
                'build', '-t', args.dockerfile_image, '-f', args.dockerfile, '.'
            ],
        )
    ])
  if args.inline_config:
    trigger.build = cloudbuild_util.LoadMessageFromPath(args.inline_config,
                                                        messages.Build,
                                                        'inline build config',
                                                        ['substitutions'])

  if args.update_substitutions:
    trigger.substitutions = cloudbuild_util.EncodeUpdatedTriggerSubstitutions(
        old_trigger.substitutions, args.update_substitutions, messages)

  if args.clear_substitutions:
    trigger.substitutions = cloudbuild_util.EncodeEmptyTriggerSubstitutions(
        messages)

  if args.remove_substitutions:
    trigger.substitutions = cloudbuild_util.RemoveTriggerSubstitutions(
        old_trigger.substitutions, args.remove_substitutions, messages
    )


def AddBranchPattern(parser):
  parser.add_argument(
      '--branch-pattern',
      metavar='REGEX',
      help="""\
A regular expression specifying which git branches to match.

This pattern is used as a regular expression search for any incoming pushes.
For example, --branch-pattern=foo will match "foo", "foobar", and "barfoo".
Events on a branch that does not match will be ignored.

The syntax of the regular expressions accepted is the syntax accepted by
RE2 and described at https://github.com/google/re2/wiki/Syntax.
""")


def AddTagPattern(parser):
  parser.add_argument(
      '--tag-pattern',
      metavar='REGEX',
      help="""\
A regular expression specifying which git tags to match.

This pattern is used as a regular expression search for any incoming pushes.
For example, --tag-pattern=foo will match "foo", "foobar", and "barfoo".
Events on a tag that does not match will be ignored.

The syntax of the regular expressions accepted is the syntax accepted by
RE2 and described at https://github.com/google/re2/wiki/Syntax.
""")


def AddGitRepoSource(flag_config):
  """Adds additional argparse flags to a group for git repo source options.

  Args:
    flag_config: argparse argument group. Git repo source flags will be added to
      this group.
  """
  flag_config.add_argument(
      '--github-enterprise-config',
      help="""\
The resource name of the GitHub Enterprise config that should be applied to
this source.
Format: projects/{project}/locations/{location}/githubEnterpriseConfigs/{id}
or projects/{project}/githubEnterpriseConfigs/{id}
""")
  repo_config = flag_config.add_argument_group(
      help='Flags for repository information')
  repo_config.add_argument(
      '--repo',
      required=True,
      help=(
          'URI of the repository. Currently only HTTP URIs for GitHub and Cloud'
          ' Source Repositories are supported.'
      ),
  )
  repo_config.add_argument(
      '--repo-type',
      required=True,
      help=(
          'Type of the repository. Currently only GitHub and Cloud Source'
          ' Repository types are supported.'
      ),
  )

  ref_config = repo_config.add_mutually_exclusive_group(required=True)
  ref_config.add_argument('--branch', help='Branch to build.')
  ref_config.add_argument('--tag', help='Tag to build.')


def ParseGitRepoSource(trigger, args, messages, required=False):
  """Parses git repo source flags.

  Args:
    trigger: The trigger to populate.
    args: An argparse arguments object.
    messages: A Cloud Build messages module.
    required: Whether or not the repository info is required.
  """

  # AddGitRepoSource (defined earlier in this file) adds repo and branch/tag
  # as required fields in the same argument group, so repo is set iff branch
  # or tag is also set. Therefore, we only need to check for the presence
  # of args.repo here.
  if required and not args.repo:
    raise c_exceptions.RequiredArgumentException(
        'REPO',
        '--repo is required when specifying a --dockerfile or --build-config.')

  # Repoless trigger.
  if not args.repo:
    return

  if not args.repo_type:
    raise c_exceptions.RequiredArgumentException(
        'REPO_TYPE',
        '--repo-type is required when specifying a --repo.')

  if args.branch:
    ref = 'refs/heads/' + args.branch
  else:
    ref = 'refs/tags/' + args.tag

  trigger.sourceToBuild = messages.GitRepoSource(
      uri=args.repo,
      ref=ref,
      repoType=messages.GitRepoSource.RepoTypeValueValuesEnum(args.repo_type),
      githubEnterpriseConfig=args.github_enterprise_config)

  if args.build_config:
    trigger.gitFileSource = messages.GitFileSource(
        path=args.build_config,
        uri=args.repo,
        revision=ref,
        repoType=messages.GitFileSource.RepoTypeValueValuesEnum(args.repo_type),
        githubEnterpriseConfig=args.github_enterprise_config)


def ParseRequireApproval(trigger, args, messages):
  """Parses approval required flag.

  Args:
    trigger: The trigger to populate.
    args: An argparse arguments object.
    messages: A Cloud Build messages module.
  """

  if args.require_approval:
    trigger.approvalConfig = messages.ApprovalConfig(approvalRequired=True)


def ParseIncludeLogsWithStatus(trigger, args, messages):
  """Parses include logs with status flag.

  Args:
    trigger: The trigger to populate.
    args: An argparse arguments object.
    messages: A Cloud Build messages module.
  """

  if args.include_logs_with_status:
    trigger.includeBuildLogs = (
        messages.BuildTrigger
        .IncludeBuildLogsValueValuesEnum.INCLUDE_BUILD_LOGS_WITH_STATUS)



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Fetch cluster credentials."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.container import util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.container import flags
from googlecloudsdk.core import log


NOT_RUNNING_MSG = """\
cluster {0} is not RUNNING. The kubernetes API may or may not be available. \
Check the cluster status for more information."""


def _BaseRun(args, context):
  """Base operations for `get-credentials` run command."""
  util.CheckKubectlInstalled()
  adapter = context['api_adapter']
  location_get = context['location_get']
  location = location_get(args)
  cluster_ref = adapter.ParseCluster(args.name, location)
  log.status.Print('Fetching cluster endpoint and auth data.')
  # Call DescribeCluster to get auth info and cache for next time
  cluster = adapter.GetCluster(cluster_ref)
  auth = cluster.masterAuth
  # TODO(b/70856999) Make this consistent with the checks in
  # api_lib/container/kubeconfig.py.
  missing_creds = not (auth and auth.clientCertificate and auth.clientKey)
  if missing_creds and not util.ClusterConfig.UseGCPAuthProvider():
    raise util.Error(
        'get-credentials requires `container.clusters.getCredentials`'
        ' permission on {0}'.format(cluster_ref.projectId)
    )
  if not adapter.IsRunning(cluster):
    log.warning(NOT_RUNNING_MSG.format(cluster_ref.clusterId))

  return cluster, cluster_ref


@base.ReleaseTracks(base.ReleaseTrack.GA)
class GetCredentials(base.Command):
  """Fetch credentials for a running cluster.

  {command} updates a `kubeconfig` file with appropriate credentials and
  endpoint information to point `kubectl` at a specific cluster in Google
  Kubernetes Engine.

  It takes a project and a zone as parameters, passed through by set
  defaults or flags. By default, credentials are written to `HOME/.kube/config`.
  You can provide an alternate path by setting the `KUBECONFIG` environment
  variable. If `KUBECONFIG` contains multiple paths, the first one is used.

  This command enables switching to a specific cluster, when working
  with multiple clusters. It can also be used to access a previously created
  cluster from a new workstation.

  By default, {command} will configure kubectl to automatically refresh its
  credentials using the same identity as gcloud. If you are running kubectl as
  part of an application, it is recommended to use [application default
  credentials](https://cloud.google.com/docs/authentication/production).
  To configure a `kubeconfig` file to use application default credentials, set
  the container/use_application_default_credentials
  [Cloud SDK property](https://cloud.google.com/sdk/docs/properties) to true
  before running {command}

  See [](https://cloud.google.com/kubernetes-engine/docs/kubectl) for
  kubectl documentation.
  """
  detailed_help = {
      'EXAMPLES':
          """\
          To switch to working on your cluster 'sample-cluster', run:

            $ {command} sample-cluster --zone=us-central1-f
      """,
  }

  @staticmethod
  def Args(parser):
    """Register flags for this command.

    Args:
      parser: An argparse.ArgumentParser-like object. It is mocked out in order
        to capture some information, but behaves like an ArgumentParser.
    """
    flags.AddGetCredentialsArgs(parser)

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Raises:
      util.Error: if the cluster is unreachable or not running.
    """
    cluster, cluster_ref = _BaseRun(args, self.context)
    util.ClusterConfig.Persist(cluster, cluster_ref.projectId, args.internal_ip)


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class GetCredentialsBeta(base.Command):
  """Fetch credentials for a running cluster.

  {command} updates a `kubeconfig` file with appropriate credentials and
  endpoint information to point `kubectl` at a specific cluster in Google
  Kubernetes Engine.

  It takes a project and a zone as parameters, passed through by set
  defaults or flags. By default, credentials are written to `HOME/.kube/config`.
  You can provide an alternate path by setting the `KUBECONFIG` environment
  variable. If `KUBECONFIG` contains multiple paths, the first one is used.

  This command enables switching to a specific cluster, when working
  with multiple clusters. It can also be used to access a previously created
  cluster from a new workstation.

  By default, {command} will configure kubectl to automatically refresh its
  credentials using the same identity as gcloud. If you are running kubectl as
  part of an application, it is recommended to use [application default
  credentials](https://cloud.google.com/docs/authentication/production).
  To configure a `kubeconfig` file to use application default credentials, set
  the container/use_application_default_credentials
  [Cloud SDK property](https://cloud.google.com/sdk/docs/properties) to true
  before running {command}

  See [](https://cloud.google.com/kubernetes-engine/docs/kubectl) for
  kubectl documentation.
  """
  detailed_help = {
      'EXAMPLES':
          """\
          To switch to working on your cluster 'sample-cluster', run:

            $ {command} sample-cluster --zone=us-central1-f
      """,
  }

  @staticmethod
  def Args(parser):
    """Register flags for this command."""
    flags.AddGetCredentialsArgs(parser)
    flags.AddCrossConnectSubnetworkFlag(parser)
    flags.AddPrivateEndpointFQDNFlag(parser)

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Raises:
      util.Error: if the cluster is unreachable or not running.
    """
    flags.VerifyGetCredentialsFlags(args)
    cluster, cluster_ref = _BaseRun(args, self.context)
    util.ClusterConfig.Persist(cluster, cluster_ref.projectId, args.internal_ip,
                               args.cross_connect_subnetwork,
                               args.private_endpoint_fqdn)


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class GetCredentialsAlpha(base.Command):
  """Fetch credentials for a running cluster.

  {command} updates a `kubeconfig` file with appropriate credentials and
  endpoint information to point `kubectl` at a specific cluster in Google
  Kubernetes Engine.

  It takes a project and a zone as parameters, passed through by set
  defaults or flags. By default, credentials are written to `HOME/.kube/config`.
  You can provide an alternate path by setting the `KUBECONFIG` environment
  variable. If `KUBECONFIG` contains multiple paths, the first one is used.

  This command enables switching to a specific cluster, when working
  with multiple clusters. It can also be used to access a previously created
  cluster from a new workstation.

  By default, {command} will configure kubectl to automatically refresh its
  credentials using the same identity as gcloud. If you are running kubectl as
  part of an application, it is recommended to use [application default
  credentials](https://cloud.google.com/docs/authentication/production).
  To configure a `kubeconfig` file to use application default credentials, set
  the container/use_application_default_credentials
  [Cloud SDK property](https://cloud.google.com/sdk/docs/properties) to true
  before running {command}

  See [](https://cloud.google.com/kubernetes-engine/docs/kubectl) for
  kubectl documentation.
  """
  detailed_help = {
      'EXAMPLES':
          """\
          To switch to working on your cluster 'sample-cluster', run:

            $ {command} sample-cluster --zone=us-central1-f
      """,
  }

  @staticmethod
  def Args(parser):
    """Register flags for this command.

    Args:
      parser: An argparse.ArgumentParser-like object. It is mocked out in order
        to capture some information, but behaves like an ArgumentParser.
    """
    flags.AddGetCredentialsArgs(parser)
    flags.AddCrossConnectSubnetworkFlag(parser)
    flags.AddPrivateEndpointFQDNFlag(parser)

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Raises:
      util.Error: if the cluster is unreachable or not running.
    """
    flags.VerifyGetCredentialsFlags(args)
    cluster, cluster_ref = _BaseRun(args, self.context)
    util.ClusterConfig.Persist(cluster, cluster_ref.projectId, args.internal_ip,
                               args.cross_connect_subnetwork,
                               args.private_endpoint_fqdn)



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to analyze IAM policy in the specified root asset."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.asset import client_util
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.asset import flags


@base.ReleaseTracks(base.ReleaseTrack.GA)
class AnalyzeIamPolicyGA(base.Command):
  """Analyzes IAM policies that match a request."""

  detailed_help = {
      'DESCRIPTION':
          ' Analyzes IAM policies that match a request.',
      'EXAMPLES':
          """\
          To find out which users have been granted the
          iam.serviceAccounts.actAs permission on a service account, run:

            $ {command} --organization=YOUR_ORG_ID --full-resource-name=YOUR_SERVICE_ACCOUNT_FULL_RESOURCE_NAME --permissions='iam.serviceAccounts.actAs'

          To find out which resources a user can access, run:

            $ {command} --organization=YOUR_ORG_ID --identity='user:u1@foo.com'

          To find out which roles or permissions a user has been granted on a
          project, run:

            $ {command} --organization=YOUR_ORG_ID --full-resource-name=YOUR_PROJECT_FULL_RESOURCE_NAME --identity='user:u1@foo.com'

          To find out which users have been granted the
          iam.serviceAccounts.actAs permission on any applicable resources, run:

            $ {command} --organization=YOUR_ORG_ID --permissions='iam.serviceAccounts.actAs'
      """
  }

  _API_VERSION = client_util.DEFAULT_API_VERSION

  @classmethod
  def Args(cls, parser):
    flags.AddAnalyzerParentArgs(parser)
    flags.AddAnalyzerSelectorsGroup(parser)
    flags.AddAnalyzerOptionsGroup(parser, True)
    flags.AddAnalyzerConditionContextGroup(parser)
    flags.AddAnalyzerSavedAnalysisQueryArgs(parser)

  def Run(self, args):
    client = client_util.AnalyzeIamPolicyClient(self._API_VERSION)
    return client.Analyze(args)



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The conversion workspaces command group for Database Migration Service."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base


@base.Hidden
@base.ReleaseTracks(base.ReleaseTrack.GA)
class ConversionWorkspaces(base.Group):
  """Manage Database Migration Service conversion workspaces.

  Commands for managing Database Migration Service conversion workspaces.
  """



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Supported printer types used in resource_printer."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals


CONFIG = 'config'
CSV = 'csv'
DEFAULT = 'default'
DIFF = 'diff'
DISABLE = 'disable'
FLATTENED = 'flattened'
GET = 'get'
JSON = 'json'
LIST = 'list'
MULTI = 'multi'
NONE = 'none'
OBJECT = 'object'
TABLE = 'table'
TEXT = 'text'
VALUE = 'value'
YAML = 'yaml'



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for dealing with ML locations API."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.py import list_pager
from googlecloudsdk.api_lib.util import apis
from googlecloudsdk.core import exceptions
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources


class NoFieldsSpecifiedError(exceptions.Error):
  """Error indicating that no updates were requested in a Patch operation."""


def _ParseLocation(location):
  return resources.REGISTRY.Parse(
      location,
      params={'projectsId': properties.VALUES.core.project.GetOrFail},
      collection='ml.projects.locations')


class LocationsClient(object):
  """High-level client for the AI Platform locations surface."""

  def __init__(self, client=None, messages=None):
    self.client = client or apis.GetClientInstance('ml', 'v1')
    self.messages = messages or self.client.MESSAGES_MODULE

  def Get(self, location):
    """Get details about a location."""
    location_ref = _ParseLocation(location)
    req = self.messages.MlProjectsLocationsGetRequest(
        name=location_ref.RelativeName())
    return self.client.projects_locations.Get(req)

  def List(self, project_ref):
    """List available locations for the project."""
    req = self.messages.MlProjectsLocationsListRequest(
        parent=project_ref.RelativeName())
    return list_pager.YieldFromList(
        self.client.projects_locations,
        req,
        field='locations',
        batch_size_attribute='pageSize')
